; ---------------------------------------------------------------
;- ShiftIt - by horst.schaeffer@gmx.net
;- Shift by Click  (without shift animation)
; ---------------------------------------------------------------
#title$ = "ShiftIt(1) 1.60"   ; 17 Jun 2013

#TileSize = 50                ; pixels (hight=width)
#Rows = 4 
#Columns = 6 
#Spacing = 3                  ; pixels between tiles
#Tiles = #Rows * #Columns -1  ; minus the gap 

#BackgroundColor = $15AC72 
#FontFace = "Georgia"
#FontSize = 16
#ShuffleCycle = 50            ; milliseconds
#Start$ = "Shuffle" 
#Stop$ = "Stop"
#Border = 5                   ; window padding
#ButtonHeight = 30            ; shuffle button

#TileOffset = #TileSize + #Spacing
#TopHeight  = #Border + #ButtonHeight + #Border

Global Dim tile.l(#rows,#columns) ; Tile grid to keep track of tile numbers - index: 1..

; Window x/y (pixels) from column/row number
Macro TileX(column) : #Border + (column-1) * #TileOffset  : EndMacro
Macro TileY(row)    : #TopHeight + (row-1) * #TileOffset  : EndMacro

; column/row number from x/y (pixels)
Macro GridColumn(WindowX) : ((WindowX - #Border) / #TileOffset +1)    : EndMacro
Macro GridRow(WindowY)    : ((WindowY - #TopHeight) / #TileOffset +1) : EndMacro

Structure gridpos : row.l : column.l : EndStructure
Global Gap.gridpos                         ; gap position
Gap\row = #rows : Gap\column = #columns    ; initial 

; ---------------------------------------------------------------
;- Window
; ---------------------------------------------------------------
#GridWidth  = #Columns * #TileOffset - 1*#Spacing
#GridHeight = #Rows * #TileOffset - 1*#Spacing

#WindowHeight = #TopHeight + #GridHeight + #Border 
#WindowWidth = #Border + #GridWidth + #Border

#Style = #PB_Window_SystemMenu | #PB_Window_ScreenCentered 
If OpenWindow(0,0,0,#WindowWidth,#WindowHeight,#title$,#Style) = #NUL : End : EndIf 
SetWindowColor(0,#BackgroundColor)

ShuffleButton = ButtonGadget(#PB_Any,#Border,#Border,#GridWidth,#ButtonHeight,#Start$,#PB_Button_Toggle|#BS_FLAT)
SetGadgetFont(ShuffleButton,LoadFont(1,"Arial",10,#PB_Font_Bold))

SetGadgetFont(#PB_Any,LoadFont(0,#FontFace,#FontSize))

For row = 1 To #rows   ; make tile buttons with gadget number = tile number
  y = TileY(row)
  For column = 1 To #columns 
    x = TileX(column)
    If Not (column = Gap\column And row = Gap\row) 
      TileNumber +1
      Tile(row,column) = TileNumber
      ButtonGadget(TileNumber,x,y,#TileSize,#TileSize,Str(TileNumber),#BS_FLAT)
    EndIf 
  Next 
Next 

; ---------------------------------------------------------------
;- Shift Action 
; ---------------------------------------------------------------
; direction codes:
#right = 1 : #left = -1  ; ColumnOffset, #nul means not changing column 
#down = 1  : #up = -1    ; RowOffset, #nul means not changing row 

Procedure ShiftAction(ClickedTileNumber)   ; TileNumber = gadget number
  ; offset of clicked tile from gap 
  ColumnOffset = Gap\column - GridColumn(GadgetX(ClickedTileNumber)) 
  RowOffset    = Gap\row - GridRow(GadgetY(ClickedTileNumber))
  
  If ColumnOffset And RowOffset : ProcedureReturn #False : EndIf  ; either one must be zero
  
  If ColumnOffset > 0     : ColumnStep = #right
  ElseIf ColumnOffset < 0 : ColumnStep = #left
  ElseIf RowOffset > 0    : RowStep = #down
  ElseIf RowOffset < 0    : RowStep = #up 
  Else : ProcedureReturn #False 
  EndIf

  Repeat                               ; shift tiles one by one, begining next to gap
    row = Gap\row - RowStep            ; grab tile next to gap and move to gap 
    column = Gap\column - ColumnStep   ; (note: either RowStep or ColumnStep is zero)

    TileNumber = Tile(row,column)      ; the tile to move to the gap 

    ResizeGadget(TileNumber,TileX(Gap\column),TileY(Gap\row),#PB_Ignore,#PB_Ignore) 
    
    Swap Tile(row,column),Tile(Gap\row,Gap\column)  ; swap numbers in grid table
    gap\row = row : gap\column = column             ; new gap coordinates
    
  Until TileNumber = ClickedTileNumber

  ProcedureReturn #True
EndProcedure 

; ---------------------------------------------------------------
;- Event Handling
; ---------------------------------------------------------------

Repeat
  If shuffling
    event = WindowEvent()
    If event = #NUL                                        ; ignore other events 
      Delay(#ShuffleCycle)
      While ShiftAction(Random(#Tiles-1)+1) = #NUL : Wend  ; make sure shift one
    EndIf 
  Else 
    event = WaitWindowEvent()
  EndIf 
  
  If event = #PB_Event_Gadget And EventType() = #PB_EventType_LeftClick
    clickedGadget = EventGadget() 
    If clickedGadget = ShuffleButton 
      shuffling = GetGadgetState(ShuffleButton) 
      If shuffling : caption$ = #Stop$ : Else : caption$ = #Start$ : EndIf 
      SetGadgetText(ShuffleButton,caption$)
    Else ; one of the tiles was clicked
      If ShiftAction(clickedGadget) = #NUL : MessageBeep_(0) : EndIf 
    EndIf 
  EndIf 
Until event = #PB_Event_CloseWindow

End 


; IDE Options = PureBasic 5.11 (Windows - x86)
; FirstLine = 36
; Folding = -
; EnableXP
; Executable = ShiftIt1.exe
; CompileSourceDirectory