; ---------------------------------------------------------------
;- ShiftIt - by horst.schaeffer@gmx.net
;- Animatied Shifting (drag tiles)
; ---------------------------------------------------------------
#title$ = "ShiftIt(2) 1.60"   ; 17 Jun 2013

#TileSize = 50                ; pixels (hight=width)
#Rows = 4 
#Columns = 6 
#Spacing = 3                  ; between tiles
#Tiles = #Rows * #Columns -1  ; minus gap 

#SnapThreshold = 50           ; % 
#ShuffleAniStep = 5           ; pixels

#BackgroundColor = $15AC72 
#FontFace = "Georgia"
#FontSize = 16
#ShuffleCycle = 20            ; milliseconds
#Start$ = "Shuffle" 
#Stop$ = "Stop"
#Border = 5                   ; window padding
#ButtonHeight = 30            ; shuffle button

#TileOffset = #TileSize + #Spacing
#TopHeight  = #Border + #ButtonHeight + #Border

Global Dim tile.l(#rows,#columns) ; Tile Grid to keep track of tile numbers - index: 1..

; Window x/y (pixels) from column/row number
Macro TileX(column) : #Border + (column-1) * #TileOffset  : EndMacro
Macro TileY(row)    : #TopHeight + (row-1) * #TileOffset  : EndMacro

; column/row number from x/y (pixels)
Macro GridColumn(WindowX) : ((WindowX - #Border) / #TileOffset +1)    : EndMacro
Macro GridRow(WindowY)    : ((WindowY - #TopHeight) / #TileOffset +1) : EndMacro

Structure gridpos : row.l : column.l : EndStructure
Global Gap.gridpos                         ; gap position
Gap\row = #rows : Gap\column = #columns    ; initial 

; ---------------------------------------------------------------
;- Window
; ---------------------------------------------------------------
#GridWidth  = #Columns * #TileOffset - 1*#Spacing
#GridHeight = #Rows * #TileOffset - 1*#Spacing

#WindowHeight = #TopHeight + #GridHeight + #Border 
#WindowWidth = #Border + #GridWidth + #Border

#Style = #PB_Window_SystemMenu | #PB_Window_ScreenCentered 
If OpenWindow(0,0,0,#WindowWidth,#WindowHeight,#title$,#Style) = #NUL : End : EndIf 
SetWindowColor(0,#BackgroundColor)

ShuffleButton = ButtonGadget(#PB_Any,#Border,#Border,#GridWidth,#ButtonHeight,#Start$,#PB_Button_Toggle|#BS_FLAT)
SetGadgetFont(ShuffleButton,LoadFont(1,"Arial",10,#PB_Font_Bold))

SetGadgetFont(#PB_Any,LoadFont(0,#FontFace,#FontSize))

For row = 1 To #rows   ; make tile buttons with gadget number = tile number
  y = TileY(row)
  For column = 1 To #columns 
    x = TileX(column)
    If Not (column = Gap\column And row = Gap\row) 
      TileNumber +1
      tile(row,column) = TileNumber
      ButtonGadget(TileNumber,x,y,#TileSize,#TileSize,Str(TileNumber),#BS_FLAT)
    EndIf 
  Next 
Next 

;- Shuffle speed buttons   --------------------------------------

SpeedDown = ButtonGadget(#PB_Any,#Border+#GridWidth-#TileSize,#Border,#TileSize/2,#ButtonHeight,"-",#BS_FLAT)
  GadgetToolTip(SpeedDown,"Slower")
  HideGadget(SpeedDown,1)
SpeedUp = ButtonGadget(#PB_Any,#Border+#GridWidth-#TileSize/2,#Border,#TileSize/2,#ButtonHeight,"+",#BS_FLAT)
  GadgetToolTip(SpeedUp,"Faster")
  HideGadget(SpeedUp,1)  


Macro ShuffelButtonStatus(on=#False)  
  ShuffleWidth = #GridWidth 
  caption.s = #Start$
  If on 
    ShuffleWidth = #GridWidth-#TileSize 
    caption = #Stop$
  EndIf 
  ResizeGadget(ShuffleButton,#PB_Ignore,#PB_Ignore,ShuffleWidth,#PB_Ignore)
  SetGadgetText(ShuffleButton,caption)
  HideGadget(SpeedDown,on!1)
  HideGadget(SpeedUp,on!1)  
EndMacro 

; ---------------------------------------------------------------
;- Shift Action 
; ---------------------------------------------------------------
Global ColumnStep, RowStep  ; 1: right or down, -1: left or up; 0: not in this direction
Global TilesToMove          ; during animation 
Global CurrentShiftValue    ; pixels, absolute

Macro AbsoluteValue(deltaX,deltaY)  ; produce one absolute value acc. to shift direction 
 ((deltaX)*ColumnStep) | ((deltaY)*RowStep) 
EndMacro 

Procedure StartWithTileIn(row,column)  ; set (number of) TilesToMove, direction.. 
  TilesToMove = 0
  ColumnStep = 0 : RowStep = 0
  CurrentShiftValue = 0
  
  ColumnOffset = Gap\column - column
  RowOffset    = Gap\row - row 

  If ColumnOffset = 0 Or RowOffset = 0   ; either one must be zero
    If ColumnOffset > 0     : ColumnStep = 1  : TilesToMove = ColumnOffset
    ElseIf ColumnOffset < 0 : ColumnStep = -1 : TilesToMove = -ColumnOffset
    ElseIf RowOffset > 0    : RowStep = 1     : TilesToMove = RowOffset
    ElseIf RowOffset < 0    : RowStep = -1    : TilesToMove = -RowOffset
    EndIf 
  EndIf 
EndProcedure 

Procedure MoveTile(ShiftValue)      ; absolute pixel offset from starting click point 

  If ShiftValue < 0 : ShiftValue = 0 
  ElseIf ShiftValue > #TileOffset : ShiftValue = #TileOffset 
  EndIf 
  If ShiftValue = CurrentShiftValue : ProcedureReturn : EndIf 
  CurrentShiftValue = ShiftValue
  
  column = Gap\column : row = Gap\row           ; starting at gap position
  x = TileX(column) + (ShiftValue*ColumnStep)
  y = TileY(row)    + (ShiftValue*RowStep)      ; either x or y will be zero
  
  TilesMoving = TilesToMove
  While TilesMoving     ; move tiles one by one, begining next to gap
    column - ColumnStep : x - #TileOffset*ColumnStep
    row - RowStep       : y - #TileOffset*RowStep
    
    ResizeGadget(tile(row,column),x,y,#PB_Ignore,#PB_Ignore)
    TilesMoving -1
  Wend  
EndProcedure 

Procedure SnapTile()
  If CurrentShiftValue >= #TileOffset*#SnapThreshold/100
    ShiftValue = #TileOffset             ; else #nul to snap back
  EndIf 
  MoveTile(ShiftValue)                   ; reset or full move 
  If ShiftValue                          ; else no change
    column = Gap\column : row = Gap\row  ; starting at gap 
    While TilesToMove                    ; swap tiles in grid table 
      column - ColumnStep 
      row - RowStep
      Swap tile(row,column),tile(Gap\row,Gap\column)
      gap\column = column : gap\row = row 
      TilesToMove -1
    Wend 
  Else 
    TilesToMove = 0
  EndIf 
EndProcedure 

; ---------------------------------------------------------------
;- By mouse or Random 
; ---------------------------------------------------------------

Procedure ShiftByMouse(position)   ; position from EventLParam()
  Static Starting.point
  Point.point 
  p.w = position & $FFFF      ; word 
  Point\x = p                 ; long (!)
  Point\y = position >> 16
  TileHandle = GetCapture_()            ; handle of clicked tile
  If TileHandle                         ; really on a tile? 
    ClientToScreen_(TileHandle,@point)  ; absolute screen coordinates
    ScreenToClient_(WindowID(0),@point) ; x/y in window
    If TilesToMove = #NUL   
      Starting\x = Point\x  
      Starting\y = Point\y
      StartWithTileIn(GridRow(Point\y),GridColumn(Point\x))
    Else 
      MoveTile(AbsoluteValue(Point\x-Starting\x,Point\y-Starting\y))
    EndIf 
  EndIf 
EndProcedure 

Procedure ShiftRandom()
  Static direction, shiftValue
  #horizontal = 0 : #vertical = 1  ; mode
  If TilesToMove = 0               ; starting new shift?
    direction !1                   ; change each time
    column = Gap\column :  row = Gap\row
    Repeat              ; repeat if random tile happens to be in gap
      If direction = #vertical 
        row = Random(#rows-1)+1
      Else  
        column = Random(#columns-1)+1 
      EndIf 
      StartWithTileIn(row,column)
    Until TilesToMove 
    shiftValue = 0 
  EndIf 
  ShiftValue + #ShuffleAniStep 
  MoveTile(shiftValue) 
  If CurrentShiftValue >= #TileOffset*80/100 ; *#SnapThreshold/100
    SnapTile()
  EndIf 
EndProcedure 

; ---------------------------------------------------------------
;- Event Handling
; ---------------------------------------------------------------
;ShuffelButtonStatus()
ShuffleCycle = #ShuffleCycle 

Repeat
  If shuffling
    event = WindowEvent()
    If event = #NUL 
      Delay(ShuffleCycle)
      ShiftRandom()
      Repeat : event = WindowEvent()  ; skip all events except gadgets
      Until event = #NUL Or event = #PB_Event_Gadget 
    EndIf 
  Else 
    event = WaitWindowEvent()
  EndIf 
  
  If event = #PB_Event_Gadget And EventType() = #PB_EventType_LeftClick
    Button = EventGadget()
    If Button = ShuffleButton 
      shuffling = GetGadgetState(ShuffleButton)
      ShuffelButtonStatus(shuffling)
      Snap = shuffling ! #True  
    ElseIf Button = speedUp : ShuffleCycle/2
      If ShuffleCycle < 1 : ShuffleCycle = 1 : EndIf
    ElseIf Button = speedDown : ShuffleCycle*2
      If ShuffleCycle > 1000 : ShuffleCycle = 1000 : EndIf 
    Else : Snap = #True  :                     ;  mouse released on button
    EndIf 
  
  ElseIf event = #WM_LBUTTONUP : Snap = #True  ; mouse released off button
  
  ElseIf event = #WM_MOUSEMOVE And EventwParam() & #MK_LBUTTON
    If TilesToMove Or LastPosition = 0         ; else invalid position 
      position = EventlParam()
      ShiftByMouse(position)
      LastPosition = position
    EndIf 
  EndIf 
  
  If Snap 
    SnapTile() : LastPosition = 0 : Snap = #False
  EndIf 
Until event = #PB_Event_CloseWindow

End 


; IDE Options = PureBasic 5.11 (Windows - x86)
; Folding = --
; EnableUnicode
; EnableXP
; Executable = ShiftIt2.exe
; CompileSourceDirectory